﻿package pers.SimpleAlarmClock;
import pers.SimpleAlarmClock.R;
import android.view.View;
import android.view.MotionEvent;
import android.content.Context;
import android.util.AttributeSet;
import android.content.res.TypedArray;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.PointF;
import android.graphics.Rect;
import android.graphics.Typeface;
import java.util.Calendar;
import android.util.Log;
//Класс для представления часов и минут в виде окружностей
public class ClockView extends View {
 private final static String tag = "ClockView";
 private float mOuterRadius;
 private float mInnerRadius; //(mOuterRadius - mInnerRadius) = толщина дуги 
 private float mArrowRadius; //Меньший радиус окружности
 private float mMiddleRadius; //Радиус, в котором остаётся "стрелка"
 private Paint mOuterPaint = new Paint(); 
 private Paint mInnerPaint = new Paint(); //Кисти для окружности часов и пустой области внутри + стрелок
 private PointF mCenter; //Центр рисования
 private PointF mArrowPosition; //Центр стрелок
 private int mTime;
 private Paint mTextPaint = new Paint(); //Кисть для чисел
 private int mMaxValue; //Макс. значение в круге часов или минут
 private boolean mSnap = true;
 private boolean mStartInit = false;
 private float mTouchSensitiveWidth; //Область, чувствительная к тыканью пальца
 private float mAttrWidth; //Толщина окружности

 private OnAlarmSetListener mOnAlarmSetListener; //Слушатель будильника

 public interface OnAlarmSetListener {
  public void onSetTime(ClockView v, int time);
  public void onProceedSetTime();
 }

 public ClockView (Context context) { this(context, null, 0); }

 public ClockView (Context context, AttributeSet attrs) {
  this(context, attrs, 0);
 }

 public ClockView(Context context, AttributeSet attrs, int defStyle) {
  super(context, attrs, defStyle);
  TypedArray a = context.obtainStyledAttributes(attrs, R.styleable.ClockView); //Получить xml атрибуты для ClockView
  final int cnt = a.getIndexCount();
  for (int i = 0; i < cnt; ++i) {
   int attr = a.getIndex(i);
   switch (attr) {
    case R.styleable.ClockView_width:
     mAttrWidth = a.getDimension(attr, 0);
    break;
    case R.styleable.ClockView_touchSensitiveWidth:
     mTouchSensitiveWidth = a.getDimension(attr, 0);
    break;
    case R.styleable.ClockView_radius:
     mMiddleRadius = a.getDimension(attr, 0);
    break;
    case R.styleable.ClockView_maxValue:
     mMaxValue = a.getInt(attr, 0);
    break;
    case R.styleable.ClockView_textSize:
     mTextPaint.setTextSize(a.getDimensionPixelSize(attr, 0));
     break;
   }
  }
  a.recycle();
  //Получить шрифт и установить параметры вывода текста
  Typeface t = Typeface.createFromAsset(context.getAssets(),"fonts/font.ttf");
  mTextPaint.setTypeface(t);
  mTextPaint.setColor(SimpleAlarmClock.BLACK);
  mTextPaint.setTextAlign(Paint.Align.CENTER);
  mTextPaint.setFlags(Paint.ANTI_ALIAS_FLAG);
  mOuterPaint.setColor(SimpleAlarmClock.BLACK);
  mInnerPaint.setColor(SimpleAlarmClock.WHITE);
  mOuterPaint.setFlags(Paint.ANTI_ALIAS_FLAG);
  mInnerPaint.setFlags(Paint.ANTI_ALIAS_FLAG);
  //По умолчанию ставим текущее время для ClockView
  Calendar current = Calendar.getInstance();
  current.setTimeInMillis(System.currentTimeMillis());
  mTime = current.get(mMaxValue > 24 ? Calendar.MINUTE : Calendar.HOUR);
 }

 @Override
 protected void onDraw (Canvas c) {
  init();
  c.drawCircle(mCenter.x, mCenter.y, mOuterRadius, mOuterPaint); //Сначала рисуем большую окружность  
  c.drawCircle(mCenter.x, mCenter.y, mInnerRadius, mInnerPaint); // потом маленькую  
  c.drawCircle(mArrowPosition.x, mArrowPosition.y, mArrowRadius, mInnerPaint); //Окружность, где будут ходить стрелки
  //Создать рисунок с текстом
  Rect r = new Rect();
  String text = "" + mTime;
  String zero = "0";
  text = (text.length() > 1 ? text : zero.concat(text));
  mTextPaint.getTextBounds(text, 0, text.length(), r);
  c.drawText(text, mCenter.x, mCenter.y+r.height()/2, mTextPaint);
 }

        /** Called just one time at the beginning to calculate 
         * the values of some class fields */
  private void init() {
   //Вызывается 1 раз для инициализации полей класса
   if (mStartInit) return;
   mStartInit = true;
   int w = getWidth();
   int h = getHeight();
   mOuterRadius = mMiddleRadius + mAttrWidth / 2;
   mInnerRadius = mMiddleRadius - mAttrWidth / 2;
   mArrowRadius = (float) (mAttrWidth * 0.4);
   mCenter = new PointF(w/2, h/2);
   mArrowPosition = new PointF();
   updateArrowPosition(); //Стрелки на текущее евремя
  }

  @Override 
  public boolean onTouchEvent(MotionEvent event) {
   //Обработчик тапа по экрану
   PointF p = new PointF(event.getX(), event.getY());
   if (event.getAction() == MotionEvent.ACTION_DOWN) { //Тап
    mOnAlarmSetListener.onProceedSetTime();
   }
   if (event.getAction() == MotionEvent.ACTION_DOWN || //Тап и перетаскивание 
       event.getAction() == MotionEvent.ACTION_MOVE) {
    double touchRadius = Math.sqrt((p.x-mCenter.x)*(p.x-mCenter.x)+(p.y-mCenter.y)*(p.y-mCenter.y));
    if (touchRadius > (mMiddleRadius + mTouchSensitiveWidth) || 
        touchRadius < (mMiddleRadius - mTouchSensitiveWidth)) {
     //Тап вне радиуса часов
     return false;
    }
    mArrowPosition.x = (float) (((mMiddleRadius * (p.x - mCenter.x)) / touchRadius) + mCenter.x);
    mArrowPosition.y = (float) (((mMiddleRadius * (p.y - mCenter.y)) / touchRadius) + mCenter.y);
    updateTime();
    updateArrowPosition();
    invalidate();
    return true;
   }
   if (event.getAction() == MotionEvent.ACTION_UP) { //Отпускание
    Log.d(tag, "ACTION_UP");
    if (mOnAlarmSetListener != null) { //Время установлено
     updateTime();
     updateArrowPosition();
     mOnAlarmSetListener.onSetTime(this, mTime);
    }
    return true;
   }
   if (event.getAction() == MotionEvent.ACTION_CANCEL) { //Отмена
    Log.d(tag, "CANCEL");
   }
   return false;
  }
      
  private void updateTime() { //Обновляет время
   PointF p = mArrowPosition;
   double dx = mCenter.x - p.x;
   double dy = mCenter.y - p.y;
   // Считать углы так, что 0 градусов - сверху
   double d = (Math.atan2(dy, dx) + 3 * Math.PI/2) % (Math.PI * 2);
   if (mMaxValue > 24) { //Минутный круг
    if (mSnap) {
     mTime = (int) (Math.round(d * mMaxValue / (Math.PI * 2)/5) * 5);
    } 
    else {
     mTime = (int) Math.round(d * mMaxValue / (Math.PI * 2));
    }
    if (mTime == 60) mTime = 0;
   } 
   else {  //Часовой круг
    mTime = (int) Math.round(d * mMaxValue / (Math.PI * 2));
    if (mTime == mMaxValue) mTime = 0;
   }
  }

  private void updateArrowPosition() { //Вычисляет координаты стрелки по времени
   double angle = (Math.PI * 2 * mTime) / mMaxValue;
   angle = (3*Math.PI/2 + angle) % (2 * Math.PI);
   mArrowPosition.set(mCenter.x, mCenter.y);
   mArrowPosition.offset((int) (Math.cos(angle)*mMiddleRadius), (int) (Math.sin(angle)*mMiddleRadius));
  }
    
  public void updateColors(int mainColor) { //Клик по области День/Ночь
   mOuterPaint.setColor(mainColor);
   mInnerPaint.setColor(mainColor ^ 0xffffff);
   mTextPaint.setColor(mainColor);
  }

  public void setOnAlarmSetListener(OnAlarmSetListener stl) {
   mOnAlarmSetListener = stl;
  }

  public void setTime(int time) {
   mTime = time;
   if (mTime == 0 && mMaxValue == 24) mTime = 0;
   invalidate();
  }

  public int getTime() { return mTime; }

  public void setSnap(boolean snap) { mSnap = snap; }
}